@@ -1,17 +1,16 @@
-from __future__ import absolute_import, unicode_literals
-
 import socket
 import warnings
+from unittest.mock import Mock, call, patch
 
 import pytest
+from case import ContextMock
 
 from amqp import Connection, spec
 from amqp.connection import SSLError
-from amqp.exceptions import ConnectionError, NotFound, ResourceError
-from amqp.five import items
+from amqp.exceptions import (ConnectionError, NotFound,
+                             RecoverableConnectionError, ResourceError)
 from amqp.sasl import AMQPLAIN, EXTERNAL, GSSAPI, PLAIN, SASL
 from amqp.transport import TCPTransport
-from case import ContextMock, Mock, call, patch
 
 
 class test_Connection:
@@ -317,7 +316,7 @@ class test_Connection:
         transport = self.conn.transport
         self.conn.collect()
         transport.close.assert_called_with()
-        for i, channel in items(channels):
+        for i, channel in channels.items():
             if i:
                 channel.collect.assert_called_with()
         assert self.conn._transport is None
@@ -356,6 +355,12 @@ class test_Connection:
         c2 = self.conn.channel(3, callback)
         assert c2 is c
 
+    def test_channel_when_connection_is_closed(self):
+        self.conn.collect()
+        callback = Mock(name='callback')
+        with pytest.raises(RecoverableConnectionError):
+            self.conn.channel(3, callback)
+
     def test_is_alive(self):
         with pytest.raises(NotImplementedError):
             self.conn.is_alive()
@@ -417,6 +422,11 @@ class test_Connection:
             (50, 60), 'payload', 'content',
         )
 
+    def test_on_inbound_method_when_connection_is_closed(self):
+        self.conn.collect()
+        with pytest.raises(RecoverableConnectionError):
+            self.conn.on_inbound_method(1, (50, 60), 'payload', 'content')
+
     def test_close(self):
         self.conn.collect = Mock(name='collect')
         self.conn.close(reply_text='foo', method_sig=spec.Channel.Open)
