@@ -1,4 +1,5 @@
-from __future__ import absolute_import, unicode_literals
+from unittest.mock import Mock
+from struct import pack
 
 import pytest
 
@@ -6,8 +7,6 @@ from amqp import spec
 from amqp.basic_message import Message
 from amqp.exceptions import UnexpectedFrame
 from amqp.method_framing import frame_handler, frame_writer
-from amqp.platform import pack
-from case import Mock
 
 
 class test_frame_handler:
@@ -116,7 +115,7 @@ class test_frame_writer:
         self.write.assert_called()
         memory = self.write.call_args[0][0]
         assert isinstance(memory, memoryview)
-        assert '\N{CHECK MARK}'.encode('utf-8') in memory.tobytes()
+        assert '\N{CHECK MARK}'.encode() in memory.tobytes()
         assert msg.properties['content_encoding'] == 'utf-8'
 
     def test_write_slow_unicode(self):
@@ -126,7 +125,7 @@ class test_frame_writer:
         self.write.assert_called()
         memory = self.write.call_args[0][0]
         assert isinstance(memory, bytes)
-        assert '\N{CHECK MARK}'.encode('utf-8') in memory
+        assert '\N{CHECK MARK}'.encode() in memory
         assert msg.properties['content_encoding'] == 'utf-8'
 
     def test_write_non_utf8(self):
