@@ -1,16 +1,15 @@
-from __future__ import absolute_import, unicode_literals
-
 import errno
+import os
 import socket
 import struct
+from struct import pack
+from unittest.mock import ANY, MagicMock, Mock, call, patch
 
 import pytest
 
 from amqp import transport
 from amqp.exceptions import UnexpectedFrame
-from amqp.platform import pack
 from amqp.transport import _AbstractTransport
-from case import ANY, MagicMock, Mock, call, patch
 
 SIGNED_INT_MAX = 0x7FFFFFFF
 
@@ -19,20 +18,22 @@ class DummyException(Exception):
     pass
 
 
-class MockSocket(object):
+class MockSocket:
     options = {}
 
     def __init__(self, *args, **kwargs):
-        super(MockSocket, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.connected = False
         self.sa = None
 
     def setsockopt(self, family, key, value):
-        if (family == socket.SOL_SOCKET and
-                key in (socket.SO_RCVTIMEO, socket.SO_SNDTIMEO)):
+        is_sol_socket = family == socket.SOL_SOCKET
+        is_receive_or_send_timeout = key in (socket.SO_RCVTIMEO,
+                                             socket.SO_SNDTIMEO)
+        if is_sol_socket and is_receive_or_send_timeout:
             self.options[key] = value
         elif not isinstance(value, int):
-            raise socket.error()
+            raise OSError()
         self.options[key] = value
 
     def getsockopt(self, family, key):
@@ -65,12 +66,6 @@ class test_socket_options:
         self.host = '127.0.0.1'
         self.connect_timeout = 3
         self.socket = MockSocket()
-        try:
-            import fcntl
-        except ImportError:
-            fcntl = None
-        if fcntl is not None:
-            patching('fcntl.fcntl')
         socket = patching('socket.socket')
         socket().getsockopt = self.socket.getsockopt
         socket().setsockopt = self.socket.setsockopt
@@ -95,6 +90,7 @@ class test_socket_options:
         patching('amqp.transport.TCPTransport._setup_transport')
         patching('amqp.transport.SSLTransport._write')
         patching('amqp.transport.SSLTransport._setup_transport')
+        patching('amqp.transport.set_cloexec')
 
     def test_backward_compatibility_tcp_transport(self):
         self.transp = transport.Transport(
@@ -178,7 +174,7 @@ class test_socket_options:
             self.transp.connect()
 
     def test_passing_wrong_value_options(self):
-        socket_settings = {TCP_KEEPINTVL: 'a'.encode()}
+        socket_settings = {TCP_KEEPINTVL: b'a'}
         self.transp = transport.Transport(
             self.host, self.connect_timeout,
             socket_settings=socket_settings,
@@ -187,7 +183,7 @@ class test_socket_options:
             self.transp.connect()
 
     def test_passing_value_as_string(self):
-        socket_settings = {TCP_KEEPIDLE: '5'.encode()}
+        socket_settings = {TCP_KEEPIDLE: b'5'}
         self.transp = transport.Transport(
             self.host, self.connect_timeout,
             socket_settings=socket_settings,
@@ -220,8 +216,8 @@ class test_socket_options:
         read_timeout_sec, read_timeout_usec = 0xdead, 0xbeef
         write_timeout_sec = 0x42
 
-        self.transp.read_timeout = read_timeout_sec + \
-            read_timeout_usec * 0.000001
+        read_timeout = read_timeout_sec + read_timeout_usec * 0.000001
+        self.transp.read_timeout = read_timeout
         self.transp.write_timeout = write_timeout_sec
         self.transp.connect()
 
@@ -235,7 +231,6 @@ class test_socket_options:
 
 
 class test_AbstractTransport:
-
     class Transport(transport._AbstractTransport):
 
         def _connect(self, *args):
@@ -245,10 +240,12 @@ class test_AbstractTransport:
             pass
 
     @pytest.fixture(autouse=True)
-    def setup_transport(self):
+    def setup_transport(self, patching):
         self.t = self.Transport('localhost:5672', 10)
         self.t.connect()
 
+        patching('amqp.transport.set_cloexec')
+
     def test_port(self):
         assert self.Transport('localhost').port == 5672
         assert self.Transport('localhost:5672').port == 5672
@@ -321,6 +318,7 @@ class test_AbstractTransport:
             self.t._read.return_value = b'thequickbrownfox'
             self.t._read.side_effect = on_read2
             return ret
+
         self.t._read.return_value = pack('>BHI', 1, 1, 16)
         self.t._read.side_effect = on_read1
 
@@ -341,9 +339,9 @@ class test_AbstractTransport:
         assert payload == b'read1read2'
 
     def transport_read_EOF(self):
-        for host, ssl in (('localhost:5672', False),
-                          ('localhost:5671', True),):
-            self.t = transport.Transport(host, ssl)
+        for host, is_ssl in (('localhost:5672', False),
+                             ('localhost:5671', True),):
+            self.t = transport.Transport(host, is_ssl)
             self.t.sock = Mock(name='socket')
             self.t.connected = True
             self.t._quick_recv = Mock(name='recv', return_value='')
@@ -389,7 +387,7 @@ class test_AbstractTransport:
             sock_mock.gettimeout.return_value = 3
             with self.t.having_timeout(5) as actual_sock:
                 assert actual_sock == self.t.sock
-            sock_mock.gettimeout.assert_called()
+            sock_mock.gettimeout.assert_called_with()
             sock_mock.settimeout.assert_has_calls(
                 [
                     call(5),
@@ -406,7 +404,7 @@ class test_AbstractTransport:
                 with self.t.having_timeout(5) as actual_sock:
                     assert actual_sock == self.t.sock
                     raise DummyException()
-            sock_mock.gettimeout.assert_called()
+            sock_mock.gettimeout.assert_called_with()
             sock_mock.settimeout.assert_has_calls(
                 [
                     call(5),
@@ -421,7 +419,7 @@ class test_AbstractTransport:
             sock_mock.gettimeout.return_value = 5
             with self.t.having_timeout(5) as actual_sock:
                 assert actual_sock == self.t.sock
-            sock_mock.gettimeout.assert_called()
+            sock_mock.gettimeout.assert_called_with()
             sock_mock.settimeout.assert_not_called()
 
     def test_set_timeout_ewouldblock_exc(self):
@@ -444,7 +442,6 @@ class test_AbstractTransport:
 
 
 class test_AbstractTransport_connect:
-
     class Transport(transport._AbstractTransport):
 
         def _init_socket(self, *args):
@@ -453,12 +450,7 @@ class test_AbstractTransport_connect:
     @pytest.fixture(autouse=True)
     def setup_transport(self, patching):
         self.t = self.Transport('localhost:5672', 10)
-        try:
-            import fcntl
-        except ImportError:
-            fcntl = None
-        if fcntl is not None:
-            patching('fcntl.fcntl')
+        patching('amqp.transport.set_cloexec')
 
     def test_connect_socket_fails(self):
         with patch('socket.socket', side_effect=socket.error):
@@ -468,13 +460,13 @@ class test_AbstractTransport_connect:
 
     def test_connect_socket_initialization_fails(self):
         with patch('socket.socket', side_effect=socket.error), \
-            patch('socket.getaddrinfo',
-                  return_value=[
-                      (socket.AF_INET, 1, socket.IPPROTO_TCP,
-                          '', ('127.0.0.1', 5672)),
-                      (socket.AF_INET, 1, socket.IPPROTO_TCP,
-                          '', ('127.0.0.2', 5672))
-                  ]):
+             patch('socket.getaddrinfo',
+                   return_value=[
+                       (socket.AF_INET, 1, socket.IPPROTO_TCP,
+                        '', ('127.0.0.1', 5672)),
+                       (socket.AF_INET, 1, socket.IPPROTO_TCP,
+                        '', ('127.0.0.2', 5672))
+                   ]):
             with pytest.raises(socket.error):
                 self.t.connect()
             assert self.t.sock is None and self.t.connected is False
@@ -484,9 +476,9 @@ class test_AbstractTransport_connect:
             patch('socket.getaddrinfo',
                   return_value=[
                       (socket.AF_INET, 1, socket.IPPROTO_TCP,
-                          '', ('127.0.0.1', 5672)),
+                       '', ('127.0.0.1', 5672)),
                       (socket.AF_INET, 1, socket.IPPROTO_TCP,
-                          '', ('127.0.0.2', 5672))
+                       '', ('127.0.0.2', 5672))
                   ]):
             self.t.sock = Mock()
             self.t.close()
@@ -500,9 +492,9 @@ class test_AbstractTransport_connect:
             patch('socket.getaddrinfo',
                   return_value=[
                       (socket.AF_INET, 1, socket.IPPROTO_TCP,
-                          '', ('127.0.0.1', 5672)),
+                       '', ('127.0.0.1', 5672)),
                       (socket.AF_INET, 1, socket.IPPROTO_TCP,
-                          '', ('127.0.0.2', 5672))
+                       '', ('127.0.0.2', 5672))
                   ]):
             self.t.sock = Mock()
             self.t.close()
@@ -512,13 +504,13 @@ class test_AbstractTransport_connect:
 
     def test_connect_short_curcuit_on_INET_succeed(self):
         with patch('socket.socket', return_value=MockSocket()), \
-            patch('socket.getaddrinfo',
-                  side_effect=[
-                      [(socket.AF_INET, 1, socket.IPPROTO_TCP,
-                          '', ('127.0.0.1', 5672))],
-                      [(socket.AF_INET6, 1, socket.IPPROTO_TCP,
-                          '', ('::1', 5672))]
-                  ]) as getaddrinfo:
+             patch('socket.getaddrinfo',
+                   side_effect=[
+                       [(socket.AF_INET, 1, socket.IPPROTO_TCP,
+                         '', ('127.0.0.1', 5672))],
+                       [(socket.AF_INET6, 1, socket.IPPROTO_TCP,
+                         '', ('::1', 5672))]
+                   ]) as getaddrinfo:
             self.t.sock = Mock()
             self.t.close()
             self.t.connect()
@@ -530,9 +522,9 @@ class test_AbstractTransport_connect:
             patch('socket.getaddrinfo',
                   side_effect=[
                       [(socket.AF_INET, 1, socket.IPPROTO_TCP,
-                          '', ('127.0.0.1', 5672))],
+                        '', ('127.0.0.1', 5672))],
                       [(socket.AF_INET6, 1, socket.IPPROTO_TCP,
-                          '', ('::1', 5672))]
+                        '', ('::1', 5672))]
                   ]) as getaddrinfo:
             self.t.sock = Mock()
             self.t.close()
@@ -550,19 +542,19 @@ class test_AbstractTransport_connect:
 
     def test_connect_getaddrinfo_raises_gaierror_once_recovers(self):
         with patch('socket.socket', return_value=MockSocket()), \
-            patch('socket.getaddrinfo',
-                  side_effect=[
-                      socket.gaierror,
-                      [(socket.AF_INET6, 1, socket.IPPROTO_TCP,
-                          '', ('::1', 5672))]
-                  ]):
+             patch('socket.getaddrinfo',
+                   side_effect=[
+                       socket.gaierror,
+                       [(socket.AF_INET6, 1, socket.IPPROTO_TCP,
+                         '', ('::1', 5672))]
+                   ]):
             self.t.connect()
 
     def test_connect_survives_not_implemented_set_cloexec(self):
         with patch('socket.socket', return_value=MockSocket()), \
-            patch('socket.getaddrinfo',
-                  return_value=[(socket.AF_INET, 1, socket.IPPROTO_TCP,
-                                 '', ('127.0.0.1', 5672))]):
+             patch('socket.getaddrinfo',
+                   return_value=[(socket.AF_INET, 1, socket.IPPROTO_TCP,
+                                  '', ('127.0.0.1', 5672))]):
             with patch('amqp.transport.set_cloexec',
                        side_effect=NotImplementedError) as cloexec_mock:
                 self.t.connect()
@@ -579,7 +571,6 @@ class test_AbstractTransport_connect:
 
 
 class test_SSLTransport:
-
     class Transport(transport.SSLTransport):
 
         def _connect(self, *args):
@@ -613,8 +604,8 @@ class test_SSLTransport:
         self.t._wrap_context.assert_called_with(sock, {'foo': 1}, c=2)
 
     def test_wrap_context(self):
-        with patch('ssl.create_default_context', create=True) \
-                as create_default_context:
+        with patch('ssl.create_default_context',
+                   create=True) as create_default_context:
             sock = Mock()
             self.t._wrap_context(sock, {'f': 1}, check_hostname=True, bar=3)
             create_default_context.assert_called_with(bar=3)
@@ -624,20 +615,18 @@ class test_SSLTransport:
 
     def test_wrap_socket_sni(self):
         sock = Mock()
-        with patch('ssl.wrap_socket') as mock_ssl_wrap:
+        with patch('ssl.SSLContext.wrap_socket') as mock_ssl_wrap:
             self.t._wrap_socket_sni(sock)
-            mock_ssl_wrap.assert_called_with(cert_reqs=0, certfile=None,
-                                             keyfile=None, sock=sock,
-                                             ca_certs=None, server_side=False,
-                                             ciphers=None, ssl_version=2,
+            mock_ssl_wrap.assert_called_with(sock=sock,
+                                             server_side=False,
+                                             do_handshake_on_connect=False,
                                              suppress_ragged_eofs=True,
-                                             do_handshake_on_connect=True)
+                                             server_hostname=None)
 
     def test_shutdown_transport(self):
         self.t.sock = None
         self.t._shutdown_transport()
-        self.t.sock = object()
-        self.t._shutdown_transport()
+
         sock = self.t.sock = Mock()
         self.t._shutdown_transport()
         assert self.t.sock is sock.unwrap()
@@ -654,7 +643,7 @@ class test_SSLTransport:
         self.t.sock = Mock(name='SSLSocket')
         self.t.sock.write.return_value = 2
         self.t._write('foo')
-        self.t.sock.write.assert_called()
+        self.t.sock.write.assert_called_with(ANY)
 
     def test_write_socket_closed(self):
         self.t.sock = Mock(name='SSLSocket')
@@ -682,14 +671,13 @@ class test_SSLTransport:
     def test_read_SSLError(self):
         self.t.sock = Mock(name='SSLSocket')
         self.t._quick_recv = Mock(name='recv', return_value='4')
-        self.t._quick_recv.side_effect = transport.SSLError('timed out')
+        self.t._quick_recv.side_effect = socket.timeout()
         self.t._read_buffer = MagicMock(return_value='AA')
         with pytest.raises(socket.timeout):
             self.t._read(64)
 
 
 class test_TCPTransport:
-
     class Transport(transport.TCPTransport):
 
         def _connect(self, *args):
@@ -716,3 +704,39 @@ class test_TCPTransport:
         with pytest.raises(IOError,
                            match=r'.*Server unexpectedly closed connection.*'):
             self.t._read(64)
+
+    def test_read_frame__windowstimeout(self, monkeypatch):
+        """Make sure BlockingIOError on Windows properly saves off partial reads.
+
+        See https://github.com/celery/py-amqp/issues/320
+        """
+
+        self.t._quick_recv = Mock()
+
+        self.t._quick_recv.side_effect = [
+            pack('>BHI', 1, 1, 16),
+            socket.error(
+                10035,
+                "A non-blocking socket operation could "
+                "not be completed immediately"
+            ),
+            b'thequickbrownfox',
+            b'\xce'
+        ]
+
+        monkeypatch.setattr(os, 'name', 'nt')
+        monkeypatch.setattr(errno, 'EWOULDBLOCK', 10035)
+
+        assert len(self.t._read_buffer) == 0
+
+        with pytest.raises(socket.timeout):
+            self.t.read_frame()
+
+        assert len(self.t._read_buffer) == 7
+
+        frame_type, channel, payload = self.t.read_frame()
+
+        assert len(self.t._read_buffer) == 0
+        assert frame_type == 1
+        assert channel == 1
+        assert payload == b'thequickbrownfox'
